{{h.mustache('#_multiple')}}
  {{h.mustache('count')}} {{_('records')}}
{{h.mustache('/_multiple')}}
{{h.mustache('^_multiple')}}
  {{h.mustache('#' + title)}}
    {{h.mustache(title)}}
  {{h.mustache('/' + title)}}
{{h.mustache('/_multiple')}}
