{% set taxonomy_fields = [
    'kingdom',
    'phylum',
    'class',
    'order',
    'family',
    'genus',
    'subgenus',
    'specificEpithet',
    'infraspecificEpithet',
    'taxonRank',
    'scientificNameAuthorship'
    'higherClassification',
 ] %}

    {% set overlap_cls = '' %}

    {% if overlapping_records_view %}
        {{h.mustache('#_multiple')}}

            {% set overlap_cls = 'multiple-records' %}
            <div class="point-detail-info">There are multiple records at this location. The information displayed here is for one of the records only.
              {% if overlapping_records_view %}
                <br/>
                <a target="_parent" href="{{h.mustache('_resource_url')}}?view_id={{overlapping_records_view}}&filters={{h.mustache('_overlapping_records_filters')}}">View all records at this location</a>
              {% endif %}
            </div>
        {{h.mustache('/_multiple')}}
        {{h.mustache('^_multiple')}}
            <div class="point-detail-info">
                <a target="_parent" href="{{h.mustache('_resource_url')}}/record/{{h.mustache('data._id')}}">View full record</a>
             </div>
        {{h.mustache('/_multiple')}}
    {% else %}
        <div class="point-detail-info">
            <a target="_parent" href="{{h.mustache('_resource_url')}}/record/{{h.mustache('data._id')}}">View full record</a>
         </div>
    {% endif %}

    <div class="point-detail {{ overlap_cls }}">

    <div class="point-detail-wrapper">

        <div class="point-detail-title">
          {{h.mustache('#data.scientificName')}}
            {{h.mustache('data.scientificName')}}
          {{h.mustache('/data.scientificName')}}
          {{h.mustache('^data.scientificName')}}
            Record {{h.mustache('data._id')}}
          {{h.mustache('/data.scientificName')}}
        </div>
        <div class="point-detail-tree">
            {% for taxonomy_field in taxonomy_fields %}
                {{h.mustache('#data.' + taxonomy_field)}}
                    <span class="point-detail-tree-label">{{h.dwc_field_title(taxonomy_field)}}:</span>
                    <span class="point-detail-tree-detail">{{h.mustache('data.' + taxonomy_field)}}</span>
                {{h.mustache('/data.' + taxonomy_field)}}
            {% endfor %}
        </div>

        <table>
            {% for field in fields %}
                {% if field not in taxonomy_fields + ['_id', 'scientificName'] %}
                    {{h.mustache('#data.' + field)}}
                        <tr>
                            <th>{{h.dwc_field_title(field)}}:</th>
                            <td>{{h.mustache('data.' + field)}}</td>
                        </tr>
                    {{h.mustache('/data.' + field)}}
                {% endif %}
            {% endfor %}
        </table>

    </div>

</div>
