
{% set overlap_cls = '' %}

    {% if overlapping_records_view %}
        {{h.mustache('^_multiple')}}

            {% set overlap_cls = 'multiple-records' %}

            <div class="point-detail-info">
                <a target="_parent" href="{{h.mustache('_resource_url')}}/record/{{h.mustache('data._id')}}">View full record</a>
             </div>
        {{h.mustache('/_multiple')}}

        {{h.mustache('#_multiple')}}
            <div class="point-detail-info">
              {% if overlapping_records_view %}
                <a target="_parent" href="{{h.mustache('_resource_url')}}?view_id={{overlapping_records_view}}&filters={{h.mustache('_overlapping_records_filters')}}">View all records at this location</a>
              {% endif %}
              <p>There are multiple records at this location. The information displayed here is for one of the records only.</p>
            </div>
        {{h.mustache('/_multiple')}}
    {% else %}
        <div class="point-detail-info">
            <a target="_parent" href="{{h.mustache('_resource_url')}}/record/{{h.mustache('data._id')}}">View full record</a>
         </div>
    {% endif %}

<div class="point-detail {{ overlap_cls }}">

    <div class="point-detail-wrapper">

    {{h.mustache('#data.' + title)}}
      <div class="point-detail-title">{{h.mustache('data.' + title)}}</div>
    {{h.mustache('/data.' + title)}}

    <table>
        {% for field in fields %}
            <tr>
                <th>{{field.title()}}:</th>
                <td>{{h.mustache('data.' + field)}}</td>
            </tr>
        {% endfor %}
    </table>

    </div>

</div>
