{% extends "dataviewer/base.html" %}

{% block page %}
  <div data-module="tiledmap"
       data-module-site_url="{{ h.dump_json(h.url('/', locale='default', qualified=true)) }}"
       data-module-resource = "{{ h.dump_json(resource_json) }}"
       data-module-resource_view = "{{ h.dump_json(resource_view_json) }}"
  >
  </div>

  {% asset 'tiledmap/leaflet-css' %}
  {% asset 'tiledmap/main-css' %}
  {% asset 'tiledmap/main-js' %}
{% endblock %}
